-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
local tm = 1
local bnr_broken = false
local supposed_fps = 60

local stall_warning_installed = true
local stall_warning_threshold = 16 --AoA at which the stall warning starts to sound
--local stall_warning_e = nil

local smokesystem = 0

local function onInit()
	electrics.values['cust_airspeed'] = 1
	electrics.values['accz'] = 0
	electrics.values['accz_x'] = 0
	electrics.values['accz_n'] = 0
	electrics.values['cust_yaw'] = 0

	electrics.values['throttle_input_c']=0

	input.state["throttle"].val=0.01

	electrics.values['elevators_input']=0
	electrics.values['ailerons_input']=0

	electrics.values['smokesystem'] = 0
	electrics.values['propellerblur'] = 0

	electrics.values['chute_deploy'] = 0

	--stall_warning_e = newExponentialSmoothing(5)
	--drivetrain.setShifterMode(1)

	bnr_broken = false

end

local function onReset()
	
	for k, node in pairs (v.data.nodes) do
		if node.name == "enc1" then sm_n_1=k  end
		if node.name == "enc2" then sm_n_2=k  end
	end

	onInit()

end

local function updateGFX(dt)
	if drivetrain.shifterPosition ~= 1 then drivetrain.shiftToGear(1) end
	--if drivetrain.shifterMode ~= 3 drivetrain.setShifterMode(3)
	

	tm = bullettime['simulationSpeed']/100

	input.state["throttle"].val=math.min(1,math.max(0,input.state["throttle"].val+0.035*electrics.values['throttle_input_c']))
	
	for n,wd in pairs(drivetrain.wheels) do
			if wd.name == "Prop" then
				rpm=wd.angularVelocity*-1
			end
	end
	if rpm then drivetrain.rpm = math.max(0, rpm* 9.5493) end
	
	if (drivetrain.rpm*tm) > (supposed_fps*60/3) then
		electrics.values['propellerblur']=1
	else
		electrics.values['propellerblur']=0
	end

	--ias = ((obj:getDirectionVector():dot(obj:getVelocity() - obj:getFlow())) * math.sqrt(obj:getDensityatNode(0) / 1.225)) * 1.94384449
	ias = ((obj:getDirectionVector():dot(obj:getVelocity() - obj:getFlow()))) * 1.94384449
	
	-- yeah, quite a shitty approach for this
	if ias <= 40 then
      electrics.values['cust_airspeed'] = ias * 20 / 40
    elseif ias <=60 then
      electrics.values['cust_airspeed'] = ((ias-40) * 30 / 20) + 20
	elseif ias <=80 then
      electrics.values['cust_airspeed'] = ((ias-60) * 40 / 20) + 50
	elseif ias <=120 then
      electrics.values['cust_airspeed'] = ((ias-80) * 70 / 40) + 90
	elseif ias <=140 then
      electrics.values['cust_airspeed'] = ((ias-120) * 40 / 20) + 160
	elseif ias <=180 then
      electrics.values['cust_airspeed'] = ((ias-140) * 60 / 40) + 200
	elseif ias <=200 then
      electrics.values['cust_airspeed'] = ((ias-180) * 25/ 20) + 260
    else
      electrics.values['cust_airspeed'] = ias - 200 + 285
    end
	
	electrics.values['accz'] = sensors['gz2'] / 9.81;
	
	if electrics.values['accz'] > electrics.values['accz_x'] then
		electrics.values['accz_x'] = electrics.values['accz']
	end
	
	if electrics.values['accz'] < electrics.values['accz_n'] then
		electrics.values['accz_n'] = electrics.values['accz']
	end
	
	electrics.values['cust_yaw'] = -1* obj:getDirection()*180/3.14
	
	if input.keys.E and not bnr_broken then
		bnr_broken=true
		for k, b in pairs(v.data.beams) do
			if b.breakGroup == 'bnr_detach' then
				obj:breakBeam(k)
				break
			end
		end
	end

	--Joystick Hydros
	electrics.values["jy_fl"] = - electrics.values['elevators_input']/20 + electrics.values['ailerons_input']/10
	electrics.values["jy_fr"] = - electrics.values['elevators_input']/20 - electrics.values['ailerons_input']/10
	electrics.values["jy_bl"] =   electrics.values['elevators_input']/20 + electrics.values['ailerons_input']/10
	electrics.values["jy_br"] =   electrics.values['elevators_input']/20 - electrics.values['ailerons_input']/10

	--Stall Warning (Based on AoA)
    if stall_warning_installed and obj:getAirSpeed() > 5 and v.userPartConfig.aerobatic_stall_warning ~= "none" then
	    vv = obj:getVelocity() - obj:getFlow()
	    vv:normalize()
	    a = math.asin(math.min(1, math.max(0, (obj:getDirectionVector()-vv).z)))/math.pi*180 --angle of attack
	    if a>stall_warning_threshold then
	    	electrics.values['stall_warning']=1
	    else
	    	electrics.values['stall_warning']=0
	    end
	    --electrics.values['stall_warning'] = math.min(1, math.max(0, stall_warning_e:get((a - stall_warning_threshold)*0.5)))
	else
		electrics.values['stall_warning'] = 0
	end
	
	--Smoke System
	if electrics.values['smokesystem'] > 0.5 and not drivetrain.engineDisabled then
		obj:addParticleByNodesRelative(sm_n_2, sm_n_1, 15+(0.05*electrics.values["throttle"]), 6, 0.1, 5)
	end
end

local function toggleEvent(s)
	if electrics.values[s] > 0.5 then
        electrics.values[s] = 0
    else
        electrics.values[s] = 1
    end
end

-- public interface
M.onInit      = onInit
M.onReset     = onReset
M.updateGFX = updateGFX
M.toggleEvent = toggleEvent

return M